package org.openscm.kundo.factory

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.apache.tools.ant.BuildException
import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.openscm.kundo.strategy.LoadDependenciesStrategy
import org.openscm.kundo.strategy.MavenLoadDependenciesStrategy
import org.openscm.kundo.strategy.EclipseLoadDependenciesStrategy
import org.openscm.kundo.strategy.KundoLoadDependenciesStrategy
/**
 * Load Dependencies Factory
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: The LoadDependenciesFactory identifies and returns an appropriate LoadDependenciesStrategy based on the build context. 
 * 
 * <p>The LoadDependenciesFactory identifies an appropriate LoadDependenciesStrategy from from data within the BuildContext. 
 * If a pom file is available in the build context pom.location property then a MavenLoadDependenciesStrategy will be returned. 
 * If no strategy can be identified then an Exception will be thrown</p>
 */
public class LoadDependenciesFactory extends AbstractDependenciesFactory{
	
	/**
	 * Constructor sets ant instance
	 * @param ant			AntBuilder instance
	 */
	 LoadDependenciesFactory( AntBuilder ant ){
		super( ant )
	}

	/**
	 * Identifies and returns an appropriate LoadDependenciesStrategy for the current state. 
	 * @param buildContext	The current BuildContext instance containing the global ant project.
	 * @return LoadDependenciesStrategy	Instance of the appropriate LoadDependenciesStrategy for the current build context data.
	 */
	 LoadDependenciesStrategy getLoadDependenciesStrategy( BuildContext buildContext ) {

		// ---------------------------------------------------
		// Maven Strategy
		// ---------------------------------------------------		
		// Check to see if the pom file exists by retrieving and loading it from the build context.
		if( strategy != null && strategy.equals( "maven" ) && buildContext.isSet( "pom.location" ) ) {
			def pom = buildContext.get( "pom.location" )
			log.debug( 'Creating Maven Install Deploy Strategy' )
			File file = new File( pom )
			if ( file.exists() ) {
				LoadDependenciesStrategy mlds = new MavenLoadDependenciesStrategy( ant, buildContext )
				return mlds
			}
		}
		
		// ---------------------------------------------------
		// Eclipse Strategy		
		// ---------------------------------------------------

		// Check to see if the property.
		if( strategy != null && strategy.indexOf( "eclipse" ) != -1 ) {
			log.debug( 'Creating Eclipse Load Dependencies Strategy' )
			LoadDependenciesStrategy elds = new EclipseLoadDependenciesStrategy( ant, buildContext )
			return elds
		}
		
		// ---------------------------------------------------
		// Default Strategy		
		// ---------------------------------------------------

		// Check to see if the property is null if so we are using Kundo.		
		if( strategy == null ){
			log.debug( 'Creating Kundo Load Dependencies Strategy' )
			LoadDependenciesStrategy klds = new KundoLoadDependenciesStrategy( ant, buildContext )
			return klds			
		}
		
		throw new BuildException( "No relevant LoadDependenciesStrategy could be found, please check your recipe and properties files to verify your config" )
	}
}
