package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.apache.tools.ant.*

 /**
 * Abstract Dependencies Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Sets up common dependency management elements for use by several delegates
 * 
 */
abstract class AbstractDependenciesStrategy {
	
	protected static final  SUPPORTED_SCOPES = [ "compile", "runtime" , "test" ]
	protected AntBuilder ant
	protected BuildContext buildContext
	final SimpleLogger log

	
	/**
	 * Constructor
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 AbstractDependenciesStrategy( AntBuilder ant, BuildContext buildContext ){
		this.ant = ant
		this.buildContext = buildContext
		this.log = SimpleLogger.getInstance( ant )
	}
	

	/**
	 * returns a path separated string of file locations from the directories
	 * specified in the properties file
	 * @param indexedProps a Map of indexed properties the interesting ones being the classpath properties
	 */
	String getPropertiesClasspath( Map indexedProps ){
		
		def extraPath = ""
		indexedProps.each{ key, value ->
			if( key == "classpath" ){
				value.each{ propertyObject ->
					if( propertyObject.recurse == null ){
						propertyObject."recurse" = "false"
					}else if( propertyObject.recurse == "" ){
						propertyObject.recurse = "false"
					}				
					if( propertyObject.jardir != null && propertyObject.jardir != "" ){
						extraPath += createPath( propertyObject.jardir, "*.jar", propertyObject.recurse )
					}
					if( propertyObject.classdir != null && propertyObject.classdir != "" ){
						extraPath += createPath( propertyObject.classdir, "*.class", propertyObject.recurse )
					}					
				}
			}
		}
		
		return extraPath
	}
	
	/**
	 * Method to create a String of pathSeparated file locations based on a base directory and an include pattern with optional recursion
	 * @param location String representation of a directory on the file system
	 * @param include String representation of an include pattern
	 * @param recurse String representation of whether directory should be searched recursively 
	 * @return String a String of pathSeparated file locations
	 */
	String createPath( String location, String include, String recurse ){
		
		def returnValue = ""
		File artifactDir = new File( location )
		if( !artifactDir.exists() ){
			return returnValue
		}
		if( !artifactDir.isDirectory() ){
			return artifactDir.getAbsolutePath()
		}
		if( include == "**" || include == "*.*" ){
			include = "all"
		}
		include = include.replace( "*", "" )		
		artifactDir.listFiles().each(){ artifact ->
			if( artifact.getAbsolutePath().endsWith( include ) || include == "all" ){
				if( artifact.isDirectory() && recurse == "true" ){
					returnValue += createPath( artifact.getAbsolutePath(), include )
				}else{
					returnValue += "${artifact.getAbsolutePath()}${File.pathSeparator}"
				}
			}else if( artifact.isDirectory() && recurse == "true" ){
				returnValue += createPath( artifact.getAbsolutePath(), include )
			}
		}		
		return returnValue
	}	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */
	void doCheck(){
		
	}
	
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	abstract void doExecute()	
}
