package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import groovy.xml.NamespaceBuilder
import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.apache.tools.ant.*

 /**
 * Abstract Load Dependencies Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Sets up common dependency management elements for use by several delegates
 * 
 */
abstract class AbstractKundoDependenciesStrategy extends AbstractDependenciesStrategy {
	
	protected def mvn
	protected def recipe
	
	/**
	 * Constructor
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 AbstractKundoDependenciesStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext )
		this.mvn = NamespaceBuilder.newInstance( ant, 'antlib:org.apache.maven.artifact.ant' )
	}
	

	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */
	void doCheck(){
		
		assert buildContext.isSet( "recipe.hierarchy" )
		
	}
	
	def getRepositoryList = { mainRecipe ->
	
		def repositoryList = mainRecipe.getRepositoryList()
		
		if( repositoryList == null ){
			if( mainRecipe.getParent() != null ){
				repositoryList = getRepositoryList( mainRecipe.getParent().getRecipe() )
			}
		}
		
		return repositoryList
	}
	
	 	
	//set up a recursive closure to return a List dependencies from all levels of parents 
	def getEntireDependencyList = { mainRecipe ->
		
		def dependencies = []
		if( mainRecipe.getParent() != null ){
			dependencies = getEntireDependencyList( mainRecipe.getParent().getRecipe() )
		}
		
		if( mainRecipe.getDependencyList() != null ){
			dependencies.addAll( mainRecipe.getDependencyList() )
		}			
		return dependencies
	}	
	
	void installDeploy( String artifactLocation, String pomLocation, String installDeploy ){
		
		def repoList
		if( installDeploy.equalsIgnoreCase( 'deploy' ) ){
			repoList = getRepositoryList( recipe )
		}
		
		def	artifactPolicy
		if( buildContext.isSet( "artifact.release.version" ) ){
			artifactPolicy = "release" 
		}else{
			artifactPolicy = "snapshot"
		}
		
		def reposToRemove = []
		if( repoList != null && !repoList.isEmpty() ){
			repoList.eachWithIndex{ repo, i ->
				if( !repo.getPolicy().equals( artifactPolicy ) ){
					reposToRemove.add( i )
				}
			}
		}
		
		if( !reposToRemove.isEmpty() ){
			reposToRemove.each{ removal ->
				if( repoList != null && !repoList.isEmpty() ){
					repoList.remove( removal )
				}
			}
		}
		
		if( repoList != null && !repoList.isEmpty()){
			repoList.each{ repo ->				
				if( repo != null && repo.getWagon() != null ){
					def wagon = repo.getWagon()
					mvn."install-provider"( groupId : wagon.getGroupId(), artifactId : wagon.getArtifactId(), version : wagon.getVersion() )
				}
			}			
		}	
		
		if( artifactLocation == null || artifactLocation == "" ){
			File pomFile = new File( pomLocation )
			if( repoList != null ){	
				repoList.each{ repo ->
					mvn."$installDeploy"() {
						mvn.pom ( file : pomLocation )	
						mvn.remoteRepository( url : repo.getDistribute() ){
							if( repo.getUsername() != null && repo.getPassword() != null ){
								mvn.authentication( username : repo.getUsername(), password : repo.getPassword() )
							}
						}
					}
				}
			}
 	 	} else {
			File pomFile = new File( pomLocation )
			File artifact = new File( artifactLocation )
			if( pomFile.exists() && artifact.exists() ){
				if( repoList != null ){
					repoList.each{ repo ->
						mvn."$installDeploy"( file : artifactLocation ){
							mvn.pom( file : pomLocation )						
							mvn.remoteRepository( url : repo.getDistribute() ){
								if( repo.getUsername() != null && repo.getPassword() != null ){
									mvn.authentication( username : repo.getUsername(), password : repo.getPassword() )
								}
							}
						}
					}					
				}
			}else{
				throw new BuildException( "pom file does not exist at this location: $pomLocation or artifact does not exist at this location: $artifactLocation" )
			}
		}
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	abstract void doExecute()	
}
