package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger

 /**
 * Eclipse Load Dependencies Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to create scoped paths from the dependencies of a project 
 * specified in eclipse metatdata.
 * 
 * <p>The eclipse load dependencies strategy reads dependency information from an eclipse .classpath file at a location set by
 * the kundo-project-plugin. Using that information it generates a separate path for each scope support by the 
 * ant4eclipse Ant Tasks (currently: compile,runtime ). It the sets that path reference in the ant
 * project with the name of the artifact being built suffixed with the scope and .classpath. This can then be 
 * retrieved at any time using the Build Context.
 * 
 * To turn on the eclipse strategies across the build use the project.helper.strategy=eclipse property
 * 
 * If your eclipse project makes use of classpath containers (e.g. JRE System Library)
 * then properties will need to be specified to resolve these in the build.properties
 * file for your project. An example for the JRE System Library would be
 * 
 * dependency-management.classpath.0.jardir=${java.home}/lib
 * dependency-management.classpath.0.recurse=true
 * 
 * This would recursively search the lib folder of your java install for jar files and add them to the classpath.</p>
 * 
 */
/* 
 * @ant-target doLoadDependencies
 */
class EclipseLoadDependenciesStrategy extends AbstractEclipseDependenciesStrategy implements LoadDependenciesStrategy {
	
	 private static final String DEFAULT_ARTIFACT_LOCATION = "target"
	 private String ps = File.pathSeparator
	 private String fs = File.separator
	/**
	 * Constructor
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 EclipseLoadDependenciesStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	 }
	
	/*
	 * A set of extra classpath directories, either classdir or jardir, depending on the property all classes or jars in the directory will be loaded to the classpath. Format: [plugin].[key].[index].[fieldname]
	 * @indexed-ant-property classpath
	 */	
	Map indexedProps
	
	/**
	 * Setter method for member variable indexedProps
	 * @param indexedProps	Map of aggregated properties objects containing additional classpath entries.
	 */	
	void setIndexedProps( Map indexedProps ){
		this.indexedProps = indexedProps
	}	
	
	/**
	 * delegate implementation method to be called by doExecute
	 *
	 */
	void doLoadDependencies(){
		if( log.isDebugEnabled() ){
			log.debug( "Entering -EclipseLoadDependenciesStrategy.doLoadDependencies( )" )
		}
		
		// get artifactId for uniqe property names
		def artifactId = buildContext.get( "artifactid" )
		
		// use ant4eclipse to get the compile time classpath of this project
		ant.getEclipseClasspath( project:basedir, property:"${artifactId}.compile.property" )
		
		// find the referenced projects of this project to find further runtime dependencies
		ant.getReferencedProjects( project:basedir, source:"project", property:"global.${artifactId}.referenced.projects" )
		def referencedProjects = buildContext.get( "${artifactId}.referenced.projects" )
		
		// for each of the referenced projects, find it's location and search it's target folder for any jars and add them to the path
		def referencedProjectsPath = ""
		referencedProjects.split( "," ).each{ project ->
			if( project != artifactId ){
				def workspace = findWorkspace( basedir )
				ant.getOutputpath( workspace:workspace, projectName:project, property:"referenced.${project}.outputpath" )
				def outputpath = ant.project.getProperty( "referenced.${project}.outputpath" )
				def referencedProjectArtifactLocation = outputpath.substring( 0, ( outputpath.indexOf( project ) + project.length() ) + 1 )
				referencedProjectArtifactLocation += "${fs}${DEFAULT_ARTIFACT_LOCATION}"
				referencedProjectsPath += createPath( referencedProjectArtifactLocation, "*.jar" )
			}
		}
		
		// if any indexed classpath properties have been specified in build.properties, add the contents to the path
		def extraPath = getPropertiesClasspath( indexedProps )
		
		// create relevant paths in the ant project
		ant.path( id:"global.${artifactId}.extra.classpath" ){
			ant.pathelement( path:extraPath )
		}
		
		ant.path( id:"global.${artifactId}.runtime.classpath" ){
			ant.pathelement( path:referencedProjectsPath )
		}
		
		ant.path( id:"global.${artifactId}.compile.classpath" ){
			ant.pathelement( path:ant.project.getProperty( "${artifactId}.compile.property" ) )
		}
		
		ant.path( id:"global.${artifactId}.test.classpath" ){
			ant.pathelement( path:ant.project.getProperty( "${artifactId}.compile.property" ) )
		}
		
		if( log.isDebugEnabled() ){
			log.debug( "Exiting -EclipseLoadDependenciesStrategy.doLoadDependencies( )" )
		}
	}
	
	/**
	 * Method to create a String of pathSeparated file locations based on a base directory and an include pattern without recursion
	 * @param location String representation of a directory on the file system
	 * @param include String representation of an include pattern
	 * @return String a String of pathSeparated file locations
	 */	
	String createPath( String location, String include ){
		
		return createPath( location, include, "false" )
	}
	
	/**
	 * Method to create a String of pathSeparated file locations based on a base directory and an include pattern with optional recursion
	 * @param location String representation of a directory on the file system
	 * @param include String representation of an include pattern
	 * @param recurse String representation of whether directory should be searched recursively 
	 * @return String a String of pathSeparated file locations
	 */
	String createPath( String location, String include, String recurse ){
		
		def returnValue = ""
		File artifactDir = new File( location )
		if( !artifactDir.exists() ){
			return returnValue
		}
		if( !artifactDir.isDirectory() ){
			return artifactDir.getAbsolutePath()
		}
		if( include == "**" || include == "*.*" ){
			include = "all"
		}
		include = include.replace( "*", "" )		
		artifactDir.listFiles().each(){ artifact ->
			if( artifact.getAbsolutePath().endsWith( include ) || include == "all" ){
				if( artifact.isDirectory() && recurse == "true" ){
					returnValue += createPath( artifact.getAbsolutePath(), include )
				}else{
					returnValue += "${artifact.getAbsolutePath()}${ps}"
				}
			}else if( artifact.isDirectory() && recurse == "true" ){
				returnValue += createPath( artifact.getAbsolutePath(), include )
			}
		}		
		return returnValue
	}
	
	/**
	 * Method to find the workspace location, currently searches for an environment variable if that doesn't exist
	 * it looks in the current projects parent directory for a directory named .metadata if neither can be found
	 * then a MissingPropertyException will be thrown.
	 * 
	 * @param projectDir String representation of the current project's directory
	 * @return String representation of the workspace containing the current projject and it's referenced projects
	 * @throws MissingPropertyException if no workspace location can be identified
	 */
	String findWorkspace( String projectDir ){
		def workspaceLoc = System.getenv().get( "ECLIPSE_WORKSPACE_LOCATION" )
		
		if( workspaceLoc == null || workspaceLoc == "" ){
			File proj = new File ( projectDir )
			
			def projParent
			if( proj.exists() ){
				projParent = proj.getParentFile() 
			}
			
			def projParentFiles
			if( projParent != null ){
				projParentFiles = projParent.listFiles()
			}
			
			if( projParentFiles != null ){
				projParentFiles.each{ file ->
					if( file.getName().equals( ".metadata" ) ){
						workspaceLoc = projParent.getAbsolutePath()
					}
				}
			}
		}
		
		if( workspaceLoc == null || workspaceLoc == "" ){
			throw new MissingPropertyException( "Unable to find the workspace for discovery of referenced projects. Please set your ECLIPSE_WORKSPACE_LOCATION environment variable to the location of the workspace containing this project and it's referenced projects." )
		}
		
		return workspaceLoc
	}
	
	/**
	 * Implementation of abstract method from AbstractPluginTargetDelegate
	 * to hold assertions that all global and local properties are available to the plugin.
	 * This method is automatically called prior to execution.
	 */
	void doCheck(){
		assert buildContext.isSet( "artifactid" )
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doCheck()
		doLoadDependencies()
	}	
}
