package org.openscm.kundo.strategy

/*
 * Copyright (C) 2008 The Ultimate People Company Ltd ("UPCO").
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import org.openscm.kundo.plugins.context.BuildContext
import org.openscm.kundo.common.logging.SimpleLogger
import org.openscm.kundo.common.recipe.*

 /**
 * Kundo Load Dependencies Strategy
 * @author <a href="mailto:ben.leedham@upco.co.uk">Ben Leedham</a>
 * @version 1.0.0
 * 
 * Description: Delegate to create scoped paths from the dependencies of a project 
 * specified in a kundo recipe file.
 * 
 * <p>The load dependencies strategy reads dependency information from a kundo recipe.groovy file. 
 * Using that information it generates a separate path for each scope support by the 
 * Maven Ant Tasks (currently: compile,runtime,test,provided ). It the sets that path reference in the ant
 * project with the name of the artifact being built suffixed with the scope and .classpath. This can then be 
 * retrieved at any time using the Build Context.
 * 
 * This is the default strategy used by kundo, to enable another strategy use the project.helper.strategy proeprty.</p>
 * 
 */
/* 
 * @ant-target doLoadDependencies
 */
class KundoLoadDependenciesStrategy extends AbstractKundoDependenciesStrategy implements LoadDependenciesStrategy {
	 
	/**
	 * Constructor
	 * @param ant			AntBuilder instance
	 * @param buildContext	BuildContext instance
	 */
	 KundoLoadDependenciesStrategy( AntBuilder ant, BuildContext buildContext ){
		super( ant, buildContext)
	}

	/*
	 * A set of extra classpath directories, either classdir or jardir, depending on the property all classes or jars in teh cirectory will be loaded to the classpath. Format: [plugin].[key].[index].[fieldname]
	 * @indexed-ant-property classpath
	 */	
	Map indexedProps
	
	/**
	 * Setter method for member variable indexedProps
	 * @param indexedProps	Map of aggregated properties objects containing additional classpath entries.
	 */	
	void setIndexedProps( Map indexedProps ){
		this.indexedProps = indexedProps
	}
	
	/**
	 * delegate implementation method to be called by doExecute
	 *
	 */
	void doLoadDependencies(){		
		if( log.isDebugEnabled() ){
			log.debug( "Entering -KundoLoadDependenciesStrategy.doLoadDependencies( )" )
		}
		
		recipe = buildContext.get( "recipe.hierarchy" )
		def classPath
		
		//call the closure and assign it's return value to a List
		def dependencyList = getEntireDependencyList( recipe )
	
		//now create a Map and put in an empty List for each scope
		def dependencyMap = [:]		
		SUPPORTED_SCOPES.each{ scope ->
			dependencyMap.put( scope, [] )
		}
		
		//for each dependency from our recipes look at the scope and
		//add it to the appropriate list in the previously created Map
		dependencyList.each{ dependency ->	
			if( dependencyMap.get( dependency.getScope() ) != null ){
				dependencyMap.get( dependency.getScope() ).add( dependency )
			}
		}
		
		def artifactId = buildContext.get( "artifactid" )
		def repoList = getRepositoryList( recipe )
		
		//for all supported scopes get the related dependency list from the Map and add as a dependency 
		//to the Maven Ant Tasks also querying the dependency for any exclusion it may specify
		if( !dependencyList.isEmpty() ){
			SUPPORTED_SCOPES.each{ scope ->				
				mvn.dependencies( filesetId:"global.${artifactId}.${scope}.fileset", pathId:"global.${artifactId}.${scope}.classpath", useScope:"${scope}" ){
					dependencyMap.get( scope ).each{ dependency ->
					
						if( dependency.getVersion() == "version" ){
							if( buildContext.isSet( "artifact.release.version" ) ){
								dependency.setVersion( buildContext.get( "artifact.release.version" ) )
							}else{
								dependency.setVersion( buildContext.get( "artifact.development.version" ) )
							}
						}
					
						mvn.dependency( groupId : dependency.getGroupId(), artifactId : dependency.getArtifactId(), version : dependency.getVersion(), type : dependency.getType(), scope : scope ){						
							if( dependency.getExclusionList() != null ){
								dependency.getExclusionList().each{ exclusion ->
									mvn.exclusion( groupId : exclusion.getGroupId(), artifactId : exclusion.getArtifactId() )
								}
							}
						}
					}
					if ( repoList != null && !repoList.isEmpty() ){
						repoList.each{ repo ->
							mvn.remoteRepository( url : repo.getDownload() )
						}
					}					
				}
				if( (log.isDebugEnabled()) && ( buildContext.isSet( "${artifactId}.${scope}.classpath" ))){							
					classPath = buildContext.get( "${artifactId}.${scope}.classpath" )							
					log.debug( "-KundoLoadDependenciesStrategy.doLoadDependencies( ) ${scope} classpath set to: ${classPath}" )
				}
			}
			// if any indexed classpath properties have been specified in build.properties, add the contents to the path
			def extraPath = getPropertiesClasspath( indexedProps )
			
			// create relevant paths in the ant project
			ant.path( id:"global.${artifactId}.extra.classpath" ){
				ant.pathelement( path:extraPath )
			}
			//make sure that the runtime scope has all the compile scope dependencies
			ant.path( id:"global.${artifactId}.runtime.classpath" ){
				if( buildContext.isSet( "${artifactId}.runtime.classpath" ) ){
					ant.pathelement( path:buildContext.get( "${artifactId}.runtime.classpath" ) )
				}
				if( buildContext.isSet( "${artifactId}.compile.classpath" ) ){
					ant.pathelement( path:buildContext.get( "${artifactId}.compile.classpath" ) )
				}
			}
		}else{
			log.debug( "-KundoLoadDependenciesStrategy.doLoadDependencies( ) No dependencies were declared for the current project." )
		}
	
		if( log.isDebugEnabled() ){
			log.debug( "Exiting -KundoLoadDependenciesStrategy.doLoadDependencies( )" )
		}
	}
	
	/**
	 * Concrete implementation of the AbstractPluginTargetDelegate.execute() method
	 */
	void doExecute(){
		doCheck()
		doLoadDependencies()
	}
	
}